@@ -2,8 +2,8 @@
          (fill-column . 70)))
  (change-log-mode . ((add-log-time-zone-rule . t)
 		     (fill-column . 74)
-		     (bug-reference-url-format . "http://debbugs.gnu.org/%s")
+		     (bug-reference-url-format . "https://debbugs.gnu.org/%s")
 		     (mode . bug-reference)))
- (emacs-lisp-mode . ((bug-reference-url-format . "http://debbugs.gnu.org/%s")
+ (emacs-lisp-mode . ((bug-reference-url-format . "https://debbugs.gnu.org/%s")
 		     (mode . bug-reference-prog)))
  (diff-mode . ((mode . whitespace))))
