@@ -49,10 +49,10 @@
 
 ;; The custom option `tramp-gvfs-methods' contains the list of
 ;; supported connection methods.  Per default, these are "afp", "dav",
-;; "davs", "obex", "sftp" and "synce".  Note that with "obex" it might
-;; be necessary to pair with the other bluetooth device, if it hasn't
-;; been done already.  There might be also some few seconds delay in
-;; discovering available bluetooth devices.
+;; "davs", "gdrive", "obex", "sftp" and "synce".  Note that with
+;; "obex" it might be necessary to pair with the other bluetooth
+;; device, if it hasn't been done already.  There might be also some
+;; few seconds delay in discovering available bluetooth devices.
 
 ;; Other possible connection methods are "ftp" and "smb".  When one of
 ;; these methods is added to the list, the remote access for that
@@ -110,21 +110,30 @@
   (require 'custom))
 
 ;;;###tramp-autoload
-(defcustom tramp-gvfs-methods '("afp" "dav" "davs" "obex" "sftp" "synce")
+(defcustom tramp-gvfs-methods
+  '("afp" "dav" "davs" "gdrive" "obex" "sftp" "synce")
   "List of methods for remote files, accessed with GVFS."
   :group 'tramp
-  :version "25.1"
+  :version "26.1"
   :type '(repeat (choice (const "afp")
 			 (const "dav")
 			 (const "davs")
 			 (const "ftp")
+			 (const "gdrive")
 			 (const "obex")
 			 (const "sftp")
 			 (const "smb")
-			 (const "synce"))))
+			 (const "synce")))
+  :require 'tramp)
 
-;; Add a default for `tramp-default-user-alist'.  Rule: For the SYNCE
-;; method, no user is chosen.
+;; Add defaults for `tramp-default-user-alist' and `tramp-default-host-alist'.
+;;;###tramp-autoload
+(when (string-match "\\(.+\\)@\\(\\(?:gmail\\|googlemail\\)\\.com\\)"
+		    user-mail-address)
+  (add-to-list 'tramp-default-user-alist
+	       `("\\`gdrive\\'" nil ,(match-string 1 user-mail-address)))
+  (add-to-list 'tramp-default-host-alist
+	       '("\\`gdrive\\'" nil ,(match-string 2 user-mail-address))))
 ;;;###tramp-autoload
 (add-to-list 'tramp-default-user-alist '("\\`synce\\'" nil nil))
 
@@ -133,7 +142,8 @@
   "Zeroconf domain to be used for discovering services, like host names."
   :group 'tramp
   :version "23.2"
-  :type 'string)
+  :type 'string
+  :require 'tramp)
 
 ;; Add the methods to `tramp-methods', in order to allow minibuffer
 ;; completion.
@@ -385,7 +395,8 @@ completion, nil means to use always cached values for discovered
 devices."
   :group 'tramp
   :version "23.2"
-  :type '(choice (const nil) integer))
+  :type '(choice (const nil) integer)
+  :require 'tramp)
 
 (defvar tramp-bluez-discovery nil
   "Indicator for a running bluetooth device discovery.
@@ -408,11 +419,9 @@ Every entry is a list (NAME ADDRESS).")
   "The device interface of the HAL daemon.")
 
 (defconst tramp-gvfs-file-attributes
-  '("type"
+  '("name"
+    "type"
     "standard::display-name"
-    ;; We don't need this one.  It is used as delimiter in case the
-    ;; display name contains spaces, which is hard to parse.
-    "standard::icon"
     "standard::symlink-target"
     "unix::nlink"
     "unix::uid"
@@ -432,9 +441,7 @@ Every entry is a list (NAME ADDRESS).")
   "GVFS file attributes.")
 
 (defconst tramp-gvfs-file-attributes-with-gvfs-ls-regexp
-  (concat "[[:blank:]]"
-	  (regexp-opt tramp-gvfs-file-attributes t)
-	  "=\\([^[:blank:]]+\\)")
+  (concat "[[:blank:]]" (regexp-opt tramp-gvfs-file-attributes t) "=\\(.+?\\)")
   "Regexp to parse GVFS file attributes with `gvfs-ls'.")
 
 (defconst tramp-gvfs-file-attributes-with-gvfs-info-regexp
@@ -473,6 +480,7 @@ Every entry is a list (NAME ADDRESS).")
     (file-modes . tramp-handle-file-modes)
     (file-name-all-completions . tramp-gvfs-handle-file-name-all-completions)
     (file-name-as-directory . tramp-handle-file-name-as-directory)
+    (file-name-case-insensitive-p . tramp-handle-file-name-case-insensitive-p)
     (file-name-completion . tramp-handle-file-name-completion)
     (file-name-directory . tramp-handle-file-name-directory)
     (file-name-nondirectory . tramp-handle-file-name-nondirectory)
@@ -498,6 +506,7 @@ Every entry is a list (NAME ADDRESS).")
     (make-auto-save-file-name . tramp-handle-make-auto-save-file-name)
     (make-directory . tramp-gvfs-handle-make-directory)
     (make-directory-internal . ignore)
+    (make-nearby-temp-file . tramp-handle-make-nearby-temp-file)
     (make-symbolic-link . tramp-handle-make-symbolic-link)
     (process-file . ignore)
     (rename-file . tramp-gvfs-handle-rename-file)
@@ -509,6 +518,7 @@ Every entry is a list (NAME ADDRESS).")
     (shell-command . ignore)
     (start-file-process . ignore)
     (substitute-in-file-name . tramp-handle-substitute-in-file-name)
+    (temporary-file-directory . tramp-handle-temporary-file-directory)
     (unhandled-file-name-directory . ignore)
     (vc-registered . ignore)
     (verify-visited-file-modtime . tramp-handle-verify-visited-file-modtime)
@@ -532,7 +542,7 @@ Operations not mentioned here will be handled by the default Emacs primitives.")
 First arg specifies the OPERATION, second arg is a list of arguments to
 pass to the OPERATION."
   (unless tramp-gvfs-enabled
-    (tramp-user-error nil "Package `tramp-gvfs' not supported"))
+    (tramp-compat-user-error nil "Package `tramp-gvfs' not supported"))
   (let ((fn (assoc operation tramp-gvfs-file-name-handler-alist)))
     (if fn
 	(save-match-data (apply (cdr fn) args))
@@ -621,7 +631,7 @@ is no information where to trace the message.")
 
 (defun tramp-gvfs-do-copy-or-rename-file
   (op filename newname &optional ok-if-already-exists keep-date
-      preserve-uid-gid preserve-extended-attributes)
+   preserve-uid-gid preserve-extended-attributes)
   "Copy or rename a remote file.
 OP must be `copy' or `rename' and indicates the operation to perform.
 FILENAME specifies the file to copy or rename, NEWNAME is the name of
@@ -716,7 +726,7 @@ file names."
 
 (defun tramp-gvfs-handle-copy-file
   (filename newname &optional ok-if-already-exists keep-date
-	    preserve-uid-gid preserve-extended-attributes)
+   preserve-uid-gid preserve-extended-attributes)
   "Like `copy-file' for Tramp files."
   (setq filename (expand-file-name filename))
   (setq newname (expand-file-name newname))
@@ -742,14 +752,19 @@ file names."
 
 (defun tramp-gvfs-handle-delete-directory (directory &optional recursive trash)
   "Like `delete-directory' for Tramp files."
-  (when (and recursive (not (file-symlink-p directory)))
-    (mapc (lambda (file)
-	    (if (eq t (car (file-attributes file)))
-		(tramp-compat-delete-directory file recursive trash)
-	      (tramp-compat-delete-file file trash)))
-	  (directory-files
-	   directory 'full "^\\([^.]\\|\\.\\([^.]\\|\\..\\)\\).*")))
   (with-parsed-tramp-file-name directory nil
+    (if (and recursive (not (file-symlink-p directory)))
+	(mapc (lambda (file)
+		(if (eq t (tramp-compat-file-attribute-type
+			   (file-attributes file)))
+		    (tramp-compat-delete-directory file recursive trash)
+		  (tramp-compat-delete-file file trash)))
+	      (directory-files
+	       directory 'full directory-files-no-dot-files-regexp))
+      (when (directory-files directory nil directory-files-no-dot-files-regexp)
+	(tramp-error
+	 v 'file-error "Couldn't delete non-empty %s" directory)))
+
     (tramp-flush-file-property v (file-name-directory localname))
     (tramp-flush-directory-property v localname)
     (unless
@@ -795,7 +810,7 @@ file names."
 	  (tramp-gvfs-maybe-open-connection (vector method user host "/" hop)))
 	(setq localname
 	      (replace-match
-	       (tramp-get-file-property  v "/" "default-location" "~")
+	       (tramp-get-connection-property v "default-location" "~")
 	       nil t localname 1)))
       ;; Tilde expansion is not possible.
       (when (string-match "\\`\\(~[^/]*\\)\\(.*\\)\\'" localname)
@@ -834,25 +849,31 @@ file names."
 	   v "gvfs-ls" "-h" "-n" "-a"
 	   (mapconcat 'identity tramp-gvfs-file-attributes ",")
 	   (tramp-gvfs-url-file-name directory))
-	  ;; Parse output ...
+	  ;; Parse output.
 	  (with-current-buffer (tramp-get-connection-buffer v)
 	    (goto-char (point-min))
-	    (while (re-search-forward
+	    (while (looking-at
 		    (concat "^\\(.+\\)[[:blank:]]"
 			    "\\([[:digit:]]+\\)[[:blank:]]"
-			    "(\\(.+\\))[[:blank:]]"
-			    "standard::display-name=\\(.+\\)[[:blank:]]"
-			    "standard::icon=")
-		    (point-at-eol) t)
-	      (let ((item (list (cons "standard::display-name" (match-string 4))
-				(cons "type" (match-string 3))
+			    "(\\(.+?\\))"
+			    tramp-gvfs-file-attributes-with-gvfs-ls-regexp))
+	      (let ((item (list (cons "type" (match-string 3))
 				(cons "standard::size" (match-string 2))
-				(match-string 1))))
-		(while (re-search-forward
-			tramp-gvfs-file-attributes-with-gvfs-ls-regexp
-			(point-at-eol) t)
-		  (push (cons (match-string 1) (match-string 2)) item))
-		(push (nreverse item) result))
+				(cons "name" (match-string 1)))))
+		(goto-char (1+ (match-end 3)))
+		(while (looking-at
+			(concat
+			 tramp-gvfs-file-attributes-with-gvfs-ls-regexp
+			 "\\(" tramp-gvfs-file-attributes-with-gvfs-ls-regexp
+			 "\\|" "$" "\\)"))
+		  (push (cons (match-string 1) (match-string 2)) item)
+		  (goto-char (match-end 2)))
+		;; Add display name as head.
+		(push
+		 (cons (cdr (or (assoc "standard::display-name" item)
+				(assoc "name" item)))
+		       (nreverse item))
+		 result))
 	      (forward-line)))
 	  result)))))
 
@@ -868,7 +889,7 @@ file names."
 	  ;; Send command.
 	  (tramp-gvfs-send-command
 	   v "gvfs-info" (tramp-gvfs-url-file-name filename))
-	  ;; Parse output ...
+	  ;; Parse output.
 	  (with-current-buffer (tramp-get-connection-buffer v)
 	    (goto-char (point-min))
 	    (while (re-search-forward
@@ -880,6 +901,7 @@ file names."
   "Return GVFS attributes association list of FILENAME."
   (setq filename (directory-file-name (expand-file-name filename)))
   (with-parsed-tramp-file-name filename nil
+    (setq localname (tramp-compat-file-name-unquote localname))
     (if (or
 	 (and (string-match "^\\(afp\\|smb\\)$" method)
 	      (string-match "^/?\\([^/]+\\)$" localname))
@@ -998,7 +1020,8 @@ file names."
 
 (defun tramp-gvfs-handle-file-directory-p (filename)
   "Like `file-directory-p' for Tramp files."
-  (eq t (car (file-attributes (file-truename filename)))))
+  (eq t (tramp-compat-file-attribute-type
+	 (file-attributes (file-truename filename)))))
 
 (defun tramp-gvfs-handle-file-executable-p (filename)
   "Like `file-executable-p' for Tramp files."
@@ -1012,7 +1035,7 @@ file names."
     (let ((tmpfile (tramp-compat-make-temp-file filename)))
       (unless (file-exists-p filename)
 	(tramp-error
-	 v 'file-error
+	 v tramp-file-missing
 	 "Cannot make local copy of non-existing file `%s'" filename))
       (copy-file filename tmpfile 'ok-if-already-exists 'keep-time)
       tmpfile)))
@@ -1024,17 +1047,12 @@ file names."
      filename
      (with-parsed-tramp-file-name (expand-file-name directory) nil
        (with-tramp-file-property v localname "file-name-all-completions"
-         (let ((result '("./" "../"))
-	       entry)
+         (let ((result '("./" "../")))
            ;; Get a list of directories and files.
 	   (dolist (item (tramp-gvfs-get-directory-attributes directory) result)
-	     (setq entry
-		   (or ;; Use display-name if available (google-drive).
-		    ;(cdr (assoc "standard::display-name" item))
-		    (car item)))
 	     (if (string-equal (cdr (assoc "type" item)) "directory")
-		 (push (file-name-as-directory entry) result)
-	       (push entry result)))))))))
+		 (push (file-name-as-directory (car item)) result)
+	       (push (car item) result)))))))))
 
 (defun tramp-gvfs-handle-file-notify-add-watch (file-name flags _callback)
   "Like `file-notify-add-watch' for Tramp files."
@@ -1070,7 +1088,7 @@ file names."
 	;; There might be an error if the monitor is not supported.
 	;; Give the filter a chance to read the output.
 	(tramp-accept-process-output p 1)
-	(unless (memq (process-status p) '(run open))
+	(unless (tramp-compat-process-live-p p)
 	  (tramp-error
 	   v 'file-notify-error "Monitoring not supported for `%s'" file-name))
 	p))))
@@ -1200,7 +1218,9 @@ file-notify events."
 
     ;; Set file modification time.
     (when (or (eq visit t) (stringp visit))
-      (set-visited-file-modtime (nth 5 (file-attributes filename))))
+      (set-visited-file-modtime
+       (tramp-compat-file-attribute-modification-time
+	(file-attributes filename))))
 
     ;; The end.
     (when (or (eq visit t) (null visit) (stringp visit))
@@ -1213,6 +1233,7 @@ file-notify events."
 (defun tramp-gvfs-url-file-name (filename)
   "Return FILENAME in URL syntax."
   ;; "/" must NOT be hexlified.
+  (setq filename (tramp-compat-file-name-unquote filename))
   (let ((url-unreserved-chars (cons ?/ url-unreserved-chars))
 	result)
     (setq
@@ -1220,6 +1241,8 @@ file-notify events."
      (url-recreate-url
       (if (tramp-tramp-file-p filename)
 	  (with-parsed-tramp-file-name filename nil
+	    (when (string-equal "gdrive" method)
+	      (setq method "google-drive"))
 	    (when (and user (string-match tramp-user-with-domain-regexp user))
 	      (setq user
 		    (concat (match-string 2 user) ";" (match-string 1 user))))
@@ -1389,6 +1412,8 @@ ADDRESS can have the form \"xx:xx:xx:xx:xx:xx\" or \"[xx:xx:xx:xx:xx:xx]\"."
 	  (setq host (tramp-bluez-device host)))
 	(when (and (string-equal "dav" method) (string-equal "true" ssl))
 	  (setq method "davs"))
+	(when (string-equal "google-drive" method)
+	  (setq method "gdrive"))
 	(unless (zerop (length domain))
 	  (setq user (concat user tramp-prefix-domain-format domain)))
 	(unless (zerop (length port))
@@ -1400,13 +1425,13 @@ ADDRESS can have the form \"xx:xx:xx:xx:xx:xx\" or \"[xx:xx:xx:xx:xx:xx]\"."
 	   signal-name (tramp-gvfs-stringify-dbus-message mount-info))
 	  (tramp-set-file-property v "/" "list-mounts" 'undef)
 	  (if (string-equal (downcase signal-name) "unmounted")
-	      (tramp-set-file-property v "/" "fuse-mountpoint" nil)
+	      (tramp-flush-file-property v "/")
 	    ;; Set prefix, mountpoint and location.
 	    (unless (string-equal prefix "/")
 	      (tramp-set-file-property v "/" "prefix" prefix))
 	    (tramp-set-file-property v "/" "fuse-mountpoint" fuse-mountpoint)
-	    (tramp-set-file-property
-	     v "/" "default-location" default-location)))))))
+	    (tramp-set-connection-property
+	     v "default-location" default-location)))))))
 
 (when tramp-gvfs-enabled
   (dbus-register-signal
@@ -1474,6 +1499,8 @@ ADDRESS can have the form \"xx:xx:xx:xx:xx:xx\" or \"[xx:xx:xx:xx:xx:xx]\"."
 	   (setq host (tramp-bluez-device host)))
 	 (when (and (string-equal "dav" method) (string-equal "true" ssl))
 	   (setq method "davs"))
+	 (when (string-equal "google-drive" method)
+	   (setq method "gdrive"))
 	 (when (and (string-equal "synce" method) (zerop (length user)))
 	   (setq user (or (tramp-file-name-user vec) "")))
 	 (unless (zerop (length domain))
@@ -1485,12 +1512,13 @@ ADDRESS can have the form \"xx:xx:xx:xx:xx:xx\" or \"[xx:xx:xx:xx:xx:xx]\"."
 		(string-equal user (or (tramp-file-name-user vec) ""))
 		(string-equal host (tramp-file-name-host vec))
 		(string-match (concat "^" (regexp-quote prefix))
-			      (tramp-file-name-localname vec)))
+			      (tramp-file-name-unquote-localname vec)))
 	   ;; Set prefix, mountpoint and location.
 	   (unless (string-equal prefix "/")
 	     (tramp-set-file-property vec "/" "prefix" prefix))
 	   (tramp-set-file-property vec "/" "fuse-mountpoint" fuse-mountpoint)
-	   (tramp-set-file-property vec "/" "default-location" default-location)
+	   (tramp-set-connection-property
+	    vec "default-location" default-location)
 	   (throw 'mounted t)))))))
 
 (defun tramp-gvfs-mount-spec-entry (key value)
@@ -1508,10 +1536,10 @@ It was \"a(say)\", but has changed to \"a{sv})\"."
 	 (domain (tramp-file-name-domain vec))
 	 (host (tramp-file-name-real-host vec))
 	 (port (tramp-file-name-port vec))
-	 (localname (tramp-file-name-localname vec))
+	 (localname (tramp-file-name-unquote-localname vec))
 	 (share (when (string-match "^/?\\([^/]+\\)" localname)
 		  (match-string 1 localname)))
-	 (ssl (when (string-match "^davs" method) "true" "false"))
+	 (ssl (if (string-match "^davs" method) "true" "false"))
 	 (mount-spec
           `(:array
             ,@(cond
@@ -1531,6 +1559,9 @@ It was \"a(say)\", but has changed to \"a{sv})\"."
                 (list (tramp-gvfs-mount-spec-entry "type" "afp-volume")
                       (tramp-gvfs-mount-spec-entry "host" host)
                       (tramp-gvfs-mount-spec-entry "volume" share)))
+               ((string-equal "gdrive" method)
+                (list (tramp-gvfs-mount-spec-entry "type" "google-drive")
+                      (tramp-gvfs-mount-spec-entry "host" host)))
                (t
                 (list (tramp-gvfs-mount-spec-entry "type" method)
                       (tramp-gvfs-mount-spec-entry "host" host))))
@@ -1553,6 +1584,44 @@ It was \"a(say)\", but has changed to \"a{sv})\"."
 
 ;; Connection functions.
 
+(defun tramp-gvfs-get-remote-uid (vec id-format)
+  "The uid of the remote connection VEC, in ID-FORMAT.
+ID-FORMAT valid values are `string' and `integer'."
+  (with-tramp-connection-property vec (format "uid-%s" id-format)
+    (let ((method (tramp-file-name-method vec))
+	  (user (tramp-file-name-user vec))
+	  (host (tramp-file-name-host vec))
+	  (localname
+	   (tramp-get-connection-property vec "default-location" nil)))
+      (cond
+       ((and user (equal id-format 'string)) user)
+       (localname
+	(tramp-compat-file-attribute-user-id
+	 (file-attributes
+	  (tramp-make-tramp-file-name method user host localname) id-format)))
+       ((equal id-format 'integer) tramp-unknown-id-integer)
+       ((equal id-format 'string) tramp-unknown-id-string)))))
+
+(defun tramp-gvfs-get-remote-gid (vec id-format)
+  "The gid of the remote connection VEC, in ID-FORMAT.
+ID-FORMAT valid values are `string' and `integer'."
+  (with-tramp-connection-property vec (format "gid-%s" id-format)
+    (let ((method (tramp-file-name-method vec))
+	  (user (tramp-file-name-user vec))
+	  (host (tramp-file-name-host vec))
+	  (localname
+	   (tramp-get-connection-property vec "default-location" nil)))
+      (cond
+       (localname
+	(tramp-compat-file-attribute-group-id
+	 (file-attributes
+	  (tramp-make-tramp-file-name method user host localname) id-format)))
+       ((equal id-format 'integer) tramp-unknown-id-integer)
+       ((equal id-format 'string) tramp-unknown-id-string)))))
+
+(defvar tramp-gvfs-get-remote-uid-gid-in-progress nil
+  "Indication, that remote uid and gid determination is in progress.")
+
 (defun tramp-gvfs-maybe-open-connection (vec)
   "Maybe open a connection VEC.
 Does not do anything if a connection is already open, but re-opens the
@@ -1577,19 +1646,19 @@ connection if a previous connection has died for some reason."
     (let* ((method (tramp-file-name-method vec))
 	   (user (tramp-file-name-user vec))
 	   (host (tramp-file-name-host vec))
-	   (localname (tramp-file-name-localname vec))
+	   (localname (tramp-file-name-unquote-localname vec))
 	   (object-path
 	    (tramp-gvfs-object-path
 	     (tramp-make-tramp-file-name method user host ""))))
 
-      (when (and (string-equal method "smb")
-		 (string-equal localname "/"))
-	(tramp-error vec 'file-error "Filename must contain a Windows share"))
-
       (when (and (string-equal method "afp")
 		 (string-equal localname "/"))
 	(tramp-error vec 'file-error "Filename must contain an AFP volume"))
 
+      (when (and (string-equal method "smb")
+		 (string-equal localname "/"))
+	(tramp-error vec 'file-error "Filename must contain a Windows share"))
+
       (with-tramp-progress-reporter
 	  vec 3
 	  (if (zerop (length user))
@@ -1657,21 +1726,21 @@ connection if a previous connection has died for some reason."
 	       (tramp-get-file-property vec "/" "fuse-mountpoint" "") "/")
 	  (tramp-error vec 'file-error "FUSE mount denied"))
 
+	;; Set connection-local variables.
+	(tramp-set-connection-local-variables vec)
+
 	;; Mark it as connected.
 	(tramp-set-connection-property
 	 (tramp-get-connection-process vec) "connected" t))))
 
   ;; In `tramp-check-cached-permissions', the connection properties
-  ;; {uig,gid}-{integer,string} are used.  We set them to their local
-  ;; counterparts.
-  (with-tramp-connection-property
-   vec "uid-integer" (tramp-get-local-uid 'integer))
-  (with-tramp-connection-property
-   vec "gid-integer" (tramp-get-local-gid 'integer))
-  (with-tramp-connection-property
-   vec "uid-string" (tramp-get-local-uid 'string))
-  (with-tramp-connection-property
-   vec "gid-string" (tramp-get-local-gid 'string)))
+  ;; {uig,gid}-{integer,string} are used.  We set them to proper values.
+  (unless tramp-gvfs-get-remote-uid-gid-in-progress
+    (let ((tramp-gvfs-get-remote-uid-gid-in-progress t))
+      (tramp-gvfs-get-remote-uid vec 'integer)
+      (tramp-gvfs-get-remote-gid vec 'integer)
+      (tramp-gvfs-get-remote-uid vec 'string)
+      (tramp-gvfs-get-remote-gid vec 'string))))
 
 (defun tramp-gvfs-send-command (vec command &rest args)
   "Send the COMMAND with its ARGS to connection VEC.
@@ -1687,7 +1756,9 @@ COMMAND is usually a command from the gvfs-* utilities.
     (with-current-buffer (tramp-get-connection-buffer vec)
       (tramp-gvfs-maybe-open-connection vec)
       (erase-buffer)
-      (zerop (apply 'tramp-call-process vec command nil t nil args)))))
+      (or (zerop (apply 'tramp-call-process vec command nil t nil args))
+	  ;; Remove information about mounted connection.
+	  (and (tramp-flush-file-property vec "/") nil)))))
 
 
 ;; D-Bus BLUEZ functions.
@@ -1896,11 +1967,15 @@ They are retrieved from the hal daemon."
 
 ;;; TODO:
 
-;; * Host name completion via afp-server, smb-server or smb-network.
-;; * Check how two shares of the same SMB server can be mounted in
+;; * Host name completion for existing mount points (afp-server,
+;;   smb-server) or via smb-network.
+;;
+;; * Check, how two shares of the same SMB server can be mounted in
 ;;   parallel.
+;;
 ;; * Apply SDP on bluetooth devices, in order to filter out obex
 ;;   capability.
+;;
 ;; * Implement obex for other serial communication but bluetooth.
 
 ;;; tramp-gvfs.el ends here
